package client

import (
	"encoding/json"
	"errors"
	"git.selly.red/Selly-Modules/natsio"
	"git.selly.red/Selly-Modules/natsio/model"
	"git.selly.red/Selly-Modules/natsio/subject"
)

// SupplierPermission ...
type SupplierPermission struct{}

// GetSupplierPermission ...
func GetSupplierPermission() SupplierPermission {
	return SupplierPermission{}
}

func (s SupplierPermission) GetListPermission(p model.GetListPermissionRequest) (*model.GetListPermissionResponse, error) {
	msg, err := natsio.GetServer().Request(subject.SupplierPermission.GetList, toBytes(p))
	if err != nil {
		return nil, err
	}

	var r struct {
		Data  *model.GetListPermissionResponse `json:"data"`
		Error string                           `json:"error"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return nil, err
	}
	if r.Error != "" {
		return nil, errors.New(r.Error)
	}

	return r.Data, nil
}

func (s SupplierPermission) CreatePermission(p model.CreatePermissionRequest) (*model.CreatePermissionResponse, error) {
	msg, err := natsio.GetServer().Request(subject.SupplierPermission.Create, toBytes(p))
	if err != nil {
		return nil, err
	}

	var r struct {
		Data  *model.CreatePermissionResponse `json:"data"`
		Error string                          `json:"error"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return nil, err
	}
	if r.Error != "" {
		return nil, errors.New(r.Error)
	}

	return r.Data, nil
}

func (s SupplierPermission) UpdatePermission(p model.UpdatePermissionRequest) error {
	msg, err := natsio.GetServer().Request(subject.SupplierPermission.Update, toBytes(p))
	if err != nil {
		return err
	}

	var r struct {
		Error string `json:"error"`
	}

	if err = json.Unmarshal(msg.Data, &r); err != nil {
		return err
	}
	if r.Error != "" {
		return errors.New(r.Error)
	}

	return nil
}
